@@ -57,6 +57,6 @@ void __weak vpr_stat(const char *fmt, va_list ap)
 {
 }
 
-void __weak pr_stat(const char *fmt, ...)
+void __weak pr_info(const char *fmt, ...)
 {
 }
